@@ -18,12 +18,12 @@
  * along with this program; if not, see <http://www.gnu.org/licenses/>.
  */
 
-/* NOTE: This is an experimental module, not yet finished. JRA. */
-
 #include "includes.h"
+#include "smbd/smbd.h"
 #include "librpc/gen_ndr/xattr.h"
 #include "librpc/gen_ndr/ndr_xattr.h"
 #include "../lib/crypto/crypto.h"
+#include "auth.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_VFS
@@ -119,12 +119,12 @@ static NTSTATUS store_acl_blob_fsp(vfs_handle_struct *handle,
 	}
 	unbecome_root();
 	if (ret) {
-		errno = saved_errno;
 		DEBUG(5, ("store_acl_blob_fsp: setting attr failed for file %s"
 			"with error %s\n",
 			fsp_str_dbg(fsp),
-			strerror(errno) ));
-		return map_nt_error_from_unix(errno);
+			strerror(saved_errno) ));
+		errno = saved_errno;
+		return map_nt_error_from_unix(saved_errno);
 	}
 	return NT_STATUS_OK;
 }
@@ -185,12 +185,12 @@ static int connect_acl_xattr(struct vfs_handle_struct *handle,
 		return ret;
 	}
 
-        /* Ensure we have the parameters correct if we're
-         * using this module. */
-        DEBUG(2,("connect_acl_xattr: setting 'inherit acls = true' "
-                "'dos filemode = true' and "
-                "'force unknown acl user = true' for service %s\n",
-                service ));
+	/* Ensure we have the parameters correct if we're
+	 * using this module. */
+	DEBUG(2,("connect_acl_xattr: setting 'inherit acls = true' "
+		"'dos filemode = true' and "
+		"'force unknown acl user = true' for service %s\n",
+		service ));
 
         lp_do_parameter(SNUM(handle->conn), "inherit acls", "true");
         lp_do_parameter(SNUM(handle->conn), "dos filemode", "true");
@@ -204,7 +204,7 @@ static struct vfs_fn_pointers vfs_acl_xattr_fns = {
 	.opendir = opendir_acl_common,
 	.mkdir = mkdir_acl_common,
 	.rmdir = rmdir_acl_common,
-	.open = open_acl_common,
+	.open_fn = open_acl_common,
 	.create_file = create_file_acl_common,
 	.unlink = unlink_acl_common,
 	.chmod = chmod_acl_module_common,
