@@ -22,6 +22,9 @@
  */
 
 #include "includes.h"
+#include "smbd/smbd.h"
+#include "system/filesys.h"
+#include "../lib/crypto/md5.h"
 
 #undef DBGC_CLASS
 #define DBGC_CLASS DBGC_VFS
@@ -36,7 +39,7 @@ struct stream_io {
 
 static SMB_INO_T stream_inode(const SMB_STRUCT_STAT *sbuf, const char *sname)
 {
-	struct MD5Context ctx;
+	MD5_CTX ctx;
         unsigned char hash[16];
 	SMB_INO_T result;
 	char *upper_sname;
@@ -807,19 +810,8 @@ static NTSTATUS streams_xattr_streaminfo(vfs_handle_struct *handle,
 		return map_nt_error_from_unix(errno);
 	}
 
-	state.streams = NULL;
-	state.num_streams = 0;
-
-	if (!S_ISDIR(sbuf.st_ex_mode)) {
-		if (!add_one_stream(mem_ctx,
-				    &state.num_streams, &state.streams,
-				    "::$DATA", sbuf.st_ex_size,
-				    SMB_VFS_GET_ALLOC_SIZE(handle->conn, fsp,
-							   &sbuf))) {
-			return NT_STATUS_NO_MEMORY;
-		}
-	}
-
+	state.streams = *pstreams;
+	state.num_streams = *pnum_streams;
 	state.mem_ctx = mem_ctx;
 	state.handle = handle;
 	state.status = NT_STATUS_OK;
@@ -839,7 +831,8 @@ static NTSTATUS streams_xattr_streaminfo(vfs_handle_struct *handle,
 
 	*pnum_streams = state.num_streams;
 	*pstreams = state.streams;
-	return NT_STATUS_OK;
+
+	return SMB_VFS_NEXT_STREAMINFO(handle, fsp, fname, mem_ctx, pnum_streams, pstreams);
 }
 
 static uint32_t streams_xattr_fs_capabilities(struct vfs_handle_struct *handle,
@@ -1022,9 +1015,35 @@ static int streams_xattr_ftruncate(struct vfs_handle_struct *handle,
 	return 0;
 }
 
+static int streams_xattr_fallocate(struct vfs_handle_struct *handle,
+					struct files_struct *fsp,
+					enum vfs_fallocate_mode mode,
+					SMB_OFF_T offset,
+					SMB_OFF_T len)
+{
+        struct stream_io *sio =
+		(struct stream_io *)VFS_FETCH_FSP_EXTENSION(handle, fsp);
+
+	DEBUG(10, ("streams_xattr_fallocate called for file %s offset %.0f"
+		"len = %.0f\n",
+		fsp_str_dbg(fsp), (double)offset, (double)len));
+
+	if (sio == NULL) {
+		return SMB_VFS_NEXT_FALLOCATE(handle, fsp, mode, offset, len);
+	}
+
+	if (!streams_xattr_recheck(sio)) {
+		return errno;
+	}
+
+	/* Let the pwrite code path handle it. */
+	return ENOSYS;
+}
+
+
 static struct vfs_fn_pointers vfs_streams_xattr_fns = {
 	.fs_capabilities = streams_xattr_fs_capabilities,
-	.open = streams_xattr_open,
+	.open_fn = streams_xattr_open,
 	.stat = streams_xattr_stat,
 	.fstat = streams_xattr_fstat,
 	.lstat = streams_xattr_lstat,
@@ -1033,6 +1052,7 @@ static struct vfs_fn_pointers vfs_streams_xattr_fns = {
 	.unlink = streams_xattr_unlink,
 	.rename = streams_xattr_rename,
         .ftruncate = streams_xattr_ftruncate,
+        .fallocate = streams_xattr_fallocate,
 	.streaminfo = streams_xattr_streaminfo,
 };
 
