@@ -22,13 +22,16 @@
 
 
 #include "includes.h"
+#include "system/passwd.h"
 #include "utils/net.h"
+#include "../libcli/security/security.h"
+#include "passdb.h"
 
 /*********************************************************
  Figure out if the input was an NT group or a SID string.
  Return the SID.
 **********************************************************/
-static bool get_sid_from_input(DOM_SID *sid, char *input)
+static bool get_sid_from_input(struct dom_sid *sid, char *input)
 {
 	GROUP_MAP map;
 
@@ -123,7 +126,7 @@ static int net_groupmap_list(struct net_context *c, int argc, const char **argv)
 
 	/* list a single group is given a name */
 	if ( ntgroup[0] || sid_string[0] ) {
-		DOM_SID sid;
+		struct dom_sid sid;
 		GROUP_MAP map;
 
 		if ( sid_string[0] )
@@ -165,7 +168,7 @@ static int net_groupmap_list(struct net_context *c, int argc, const char **argv)
 
 static int net_groupmap_add(struct net_context *c, int argc, const char **argv)
 {
-	DOM_SID sid;
+	struct dom_sid sid;
 	fstring ntgroup = "";
 	fstring unixgrp = "";
 	fstring string_sid = "";
@@ -200,10 +203,10 @@ static int net_groupmap_add(struct net_context *c, int argc, const char **argv)
 	for ( i=0; i<argc; i++ ) {
 		if ( !StrnCaseCmp(argv[i], "rid", strlen("rid")) ) {
 			rid = get_int_param(argv[i]);
-			if ( rid < DOMAIN_GROUP_RID_ADMINS ) {
+			if ( rid < DOMAIN_RID_ADMINS ) {
 				d_fprintf(stderr,
 					  _("RID must be greater than %d\n"),
-					  (uint32)DOMAIN_GROUP_RID_ADMINS-1);
+					  (uint32)DOMAIN_RID_ADMINS-1);
 				return -1;
 			}
 		}
@@ -299,8 +302,7 @@ static int net_groupmap_add(struct net_context *c, int argc, const char **argv)
 
 	/* append the rid to our own domain/machine SID if we don't have a full SID */
 	if ( !string_sid[0] ) {
-		sid_copy(&sid, get_global_sam_sid());
-		sid_append_rid(&sid, rid);
+		sid_compose(&sid, get_global_sam_sid(), rid);
 		sid_to_fstring(string_sid, &sid);
 	}
 
@@ -336,7 +338,7 @@ static int net_groupmap_add(struct net_context *c, int argc, const char **argv)
 
 static int net_groupmap_modify(struct net_context *c, int argc, const char **argv)
 {
-	DOM_SID sid;
+	struct dom_sid sid;
 	GROUP_MAP map;
 	fstring ntcomment = "";
 	fstring type = "";
@@ -484,7 +486,7 @@ static int net_groupmap_modify(struct net_context *c, int argc, const char **arg
 
 static int net_groupmap_delete(struct net_context *c, int argc, const char **argv)
 {
-	DOM_SID sid;
+	struct dom_sid sid;
 	fstring ntgroup = "";
 	fstring sid_string = "";
 	int i;
@@ -581,7 +583,7 @@ static int net_groupmap_set(struct net_context *c, int argc, const char **argv)
 	have_map = pdb_getgrnam(&map, ntgroup);
 
 	if (!have_map) {
-		DOM_SID sid;
+		struct dom_sid sid;
 		have_map = ( (strncmp(ntgroup, "S-", 2) == 0) &&
 			     string_to_sid(&sid, ntgroup) &&
 			     pdb_getgrsid(&map, sid) );
@@ -612,8 +614,7 @@ static int net_groupmap_set(struct net_context *c, int argc, const char **argv)
 			}
 		}
 
-		sid_copy(&map.sid, get_global_sam_sid());
-		sid_append_rid(&map.sid, c->opt_rid);
+		sid_compose(&map.sid, get_global_sam_sid(), c->opt_rid);
 
 		map.sid_name_use = SID_NAME_DOM_GRP;
 		fstrcpy(map.nt_name, ntgroup);
@@ -707,7 +708,7 @@ static int net_groupmap_cleanup(struct net_context *c, int argc, const char **ar
 
 static int net_groupmap_addmem(struct net_context *c, int argc, const char **argv)
 {
-	DOM_SID alias, member;
+	struct dom_sid alias, member;
 
 	if ( (argc != 2) ||
 	     c->display_usage ||
@@ -730,7 +731,7 @@ static int net_groupmap_addmem(struct net_context *c, int argc, const char **arg
 
 static int net_groupmap_delmem(struct net_context *c, int argc, const char **argv)
 {
-	DOM_SID alias, member;
+	struct dom_sid alias, member;
 
 	if ( (argc != 2) ||
 	     c->display_usage ||
@@ -753,8 +754,8 @@ static int net_groupmap_delmem(struct net_context *c, int argc, const char **arg
 
 static int net_groupmap_listmem(struct net_context *c, int argc, const char **argv)
 {
-	DOM_SID alias;
-	DOM_SID *members;
+	struct dom_sid alias;
+	struct dom_sid *members;
 	size_t i, num;
 
 	if ( (argc != 1) ||
@@ -786,8 +787,8 @@ static int net_groupmap_listmem(struct net_context *c, int argc, const char **ar
 }
 
 static bool print_alias_memberships(TALLOC_CTX *mem_ctx,
-				    const DOM_SID *domain_sid,
-				    const DOM_SID *member)
+				    const struct dom_sid *domain_sid,
+				    const struct dom_sid *member)
 {
 	uint32 *alias_rids;
 	size_t i, num_alias_rids;
@@ -804,9 +805,8 @@ static bool print_alias_memberships(TALLOC_CTX *mem_ctx,
 	}
 
 	for (i = 0; i < num_alias_rids; i++) {
-		DOM_SID alias;
-		sid_copy(&alias, domain_sid);
-		sid_append_rid(&alias, alias_rids[i]);
+		struct dom_sid alias;
+		sid_compose(&alias, domain_sid, alias_rids[i]);
 		printf("%s\n", sid_string_tos(&alias));
 	}
 
@@ -816,7 +816,7 @@ static bool print_alias_memberships(TALLOC_CTX *mem_ctx,
 static int net_groupmap_memberships(struct net_context *c, int argc, const char **argv)
 {
 	TALLOC_CTX *mem_ctx;
-	DOM_SID *domain_sid, *builtin_sid, member;
+	struct dom_sid *domain_sid, member;
 
 	if ( (argc != 1) ||
 	     c->display_usage ||
@@ -834,14 +834,13 @@ static int net_groupmap_memberships(struct net_context *c, int argc, const char
 	}
 
 	domain_sid = get_global_sam_sid();
-	builtin_sid = string_sid_talloc(mem_ctx, "S-1-5-32");
-	if ((domain_sid == NULL) || (builtin_sid == NULL)) {
+	if (domain_sid == NULL) {
 		d_fprintf(stderr, _("Could not get domain sid\n"));
 		return -1;
 	}
 
 	if (!print_alias_memberships(mem_ctx, domain_sid, &member) ||
-	    !print_alias_memberships(mem_ctx, builtin_sid, &member))
+	    !print_alias_memberships(mem_ctx, &global_sid_Builtin, &member))
 		return -1;
 
 	talloc_destroy(mem_ctx);
